VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "FileObject"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Allows clients to search for a file"
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

'   (Cancellable) Event raised as the search progresses through folders
Event SearchStatus(ByVal CurrentFolder As String, Cancel As Boolean)
Attribute SearchStatus.VB_Description = "This event is fired whenever the search routine starts to scan a different folder."
'   Event raised when a file matching the specified spec is found
Event Found(ByVal FileInfo As File)
Attribute Found.VB_Description = "Event raised when a file matching the specified spec is found"

'   Max allowable length of a folder/filename
Private Const MAX_PATH = 260

'   The FILETIME data structure is a 64-bit value representing the number
'   of 100-nanosecond intervals since January 1, 1601. It is the
'   means by which Win32 determines the date and time.
Private Type FILETIME
   dwLowDateTime As Long
   dwHighDateTime As Long
End Type

'   structure describes a file found by the FindFirstFile, FindFirstFileEx,
'   or FindNextFile function
Private Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime As FILETIME
    ftLastAccessTime As FILETIME
    ftLastWriteTime As FILETIME
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName As String * MAX_PATH
    cAlternate As String * 14
End Type

'   searches a directory for a file whose name matches the specified file name.
'   FindFirstFile examines subdirectory names as well as filenames.
Private Declare Function apiFindFirstFile Lib "kernel32" _
    Alias "FindFirstFileA" _
    (ByVal lpFileName As String, _
    lpFindFileData As WIN32_FIND_DATA) _
    As Long

'    continues a file search from a previous call to the FindFirstFile function
Private Declare Function apiFindNextFile Lib "kernel32" _
    Alias "FindNextFileA" _
    (ByVal hFindFile As Long, _
    lpFindFileData As WIN32_FIND_DATA) _
    As Long

'   closes the specified search handle.
Private Declare Function apiFindClose Lib "kernel32" _
    Alias "FindClose" _
    (ByVal hFindFile As Long) _
    As Long

Private Declare Function apiGetFileAttributes Lib "kernel32" _
    Alias "GetFileAttributesA" _
    (ByVal lpFileName As String) As Long

'   finds the message definition in a message table resource based on a
'   message identifier and a language identifier.
Private Declare Function apiFormatMsgLong Lib "kernel32" _
    Alias "FormatMessageA" _
    (ByVal dwFlags As Long, _
    ByVal lpSource As Long, _
    ByVal dwMessageId As Long, _
    ByVal dwLanguageId As Long, _
    ByVal lpBuffer As String, _
    ByVal nSize As Long, _
    Arguments As Long) _
    As Long

'   fills a buffer with strings that specify valid drives in the system.
Private Declare Function apiGetLogicalDriveStrings Lib "kernel32" _
    Alias "GetLogicalDriveStringsA" _
    (ByVal nBufferLength As Long, _
    ByVal lpBuffer As String) _
    As Long

'   whether a disk drive is a removable, fixed, CD-ROM, RAM disk, or network drive.
Private Declare Function apiGetDriveType Lib "kernel32" _
    Alias "GetDriveTypeA" _
    (ByVal nDrive As String) _
    As Long
    
'   function compares two character strings, using the locale specified by
'   the given identifier as the basis for the comparison.
Private Declare Function apiCompareString Lib "kernel32" _
    Alias "CompareStringA" _
    (ByVal Locale As Long, _
    ByVal dwCmpFlags As Long, _
    ByVal lpString1 As Long, _
    ByVal cchCount1 As Long, _
    ByVal lpString2 As Long, _
    ByVal cchCount2 As Long) _
    As Long

'   // CompareString flags
Private Const LOCALE_SYSTEM_DEFAULT = &H800
Private Const NORM_IGNORECASE = &H1
Private Const CSTR_EQUAL = 2

'   // GetDriveType flags
Private Const DRIVE_FIXED = 3

Private Const FILE_ATTRIBUTE_DIRECTORY = &H10

Private Const INVALID_HANDLE_VALUE = -1
Private Const ERROR_NO_MORE_FILES = 18&

Private Const FORMAT_MESSAGE_FROM_SYSTEM As Long = &H1000

'   These are declared here just to avoid the allocation time
'   that VBA would have otherwise taken, had these been declared within
'   the recursive/repetetive procs.
Private mclsFiles As Files
Private hSearchFile As Long
Private lpDataFolder As WIN32_FIND_DATA
Private lpDataFile As WIN32_FIND_DATA
Private Const vbKeyDot = 46

'   The search path
Private mstrPath As String
'   Flag denotes whether all mapped drives should be searched
Private mblnLocalDrivesOnly As Boolean
'  Flag indicating whether we need to include the subfolders
Private mblnIncludeSubFolders As Boolean

Public Property Let IncludeSubFolders(Value As Boolean)
Attribute IncludeSubFolders.VB_Description = "Indicates whether the search routine will recursively go through any subfolders"
' if True, only the SearchPath will be searched
    mblnIncludeSubFolders = Value
End Property

Public Property Get CompactPath(Path As String, MaxCharacters As Integer) As String
Attribute CompactPath.VB_Description = "Trims the filename passed to it so that it's maximum width is MaxCharacters.  The trimmed space is replaced by dots.  If the system does not support this property, the property returns a zero length string"
'   returns a truncated path if Len(Path)>MaxCharacters
'       Requires IE4 to be installed under Win95/NT
'
On Error GoTo ErrHandler
    
    If mclsFiles.IEVersion > 4& Then
        CompactPath = fCompactPath(Path, MaxCharacters)
    End If

ExitHere:
    Exit Property
ErrHandler:
    CompactPath = vbNullString
    With Err
        .Raise .Number, .Source, .Description, .HelpFile, .HelpContext
    End With
    Resume ExitHere
End Property

Public Property Get FilesFound() As Files
Attribute FilesFound.VB_Description = "A collection of File objects representing all files found in a search operation."
'   Constructor for Files class, allows access to all it's methods
'
    If mclsFiles Is Nothing Then Set mclsFiles = New Files
    Set FilesFound = mclsFiles
End Property

Public Property Let LocalDrivesOnly(Value As Boolean)
Attribute LocalDrivesOnly.VB_Description = "If true, the search will be performed on local drives only."
'   If true, will cause the search to take place only in local drives
'
    mblnLocalDrivesOnly = Value
End Property

Public Property Let SearchPath(Path As String)
Attribute SearchPath.VB_Description = "Specifies the location which will be the beginning point for a search.  If missing, all drives will be included in the subsequent search operation."
'   Sets a search path.  If not specified, the class
'   may search all local drives (mblnLocalDrivesOnly) or
'   all mapped drives.
'
    mstrPath = Path
End Property

Public Sub FindFile(Optional ByVal FileName As String)
Attribute FindFile.VB_Description = "Initiates a search operation.  If a search specification is missing, all files will be returned."
'   The entry point for a search
'
On Error GoTo ErrHandler
    Dim lpBuffer As String
    Dim lngResult As Long
    Dim intPos As Integer
    Dim i As Integer
    Dim astrPaths() As String
    
    '   instantiate a new Files object every time
    If IsObject(mclsFiles) Then Set mclsFiles = Nothing
    Set mclsFiles = New Files
    
    If Len(mstrPath) = 0 Then
        '   If no search path was specified, get all drive letters
        mstrPath = Me.LocalDriveLetters()
    Else
        '   A search path was specified, just make sure that it has
        '   a trailing backslash
        If Not Asc(Right$(mstrPath, 1)) = 92 Then
            mstrPath = mstrPath & "\"
        End If
    End If
    
    '   Use an array since there could be multiple drives to search
    astrPaths = Split(mstrPath, vbNullChar)
    
    For i = LBound(astrPaths) To UBound(astrPaths)
        If mblnLocalDrivesOnly Then
            '   if we're supposed to only search in local drives
            If fIsLocalFixedDrive(astrPaths(i)) Then
                '   Get all the folders for a specified parent folder
                '   this is a recursive proc and calls sEnumFiles by itself
                '   to handle all the files in the particular folder
                Call sGetFolders(astrPaths(i), FileName)
            End If
        Else
            Call sGetFolders(astrPaths(i), FileName)
        End If
    Next
ExitHere:
    Exit Sub
ErrHandler:
    With Err
        .Raise .Number, "FileObject::FindFile", fAPIErr(.Number)
    End With
    Resume ExitHere
End Sub

Public Function LocalDriveLetters() As Variant
Attribute LocalDriveLetters.VB_Description = "Returns a null character separated string containing all mapped drive letters."
'   Returns all installed drive letters in a vbNullChar separated string
    LocalDriveLetters = fGetDrives()
End Function

Private Sub sGetFolders(strPath As String, strFindFile As String)
'   Recursive proc!!!
'   retrieves a list of all folders under the path strPath
'   and then call sEnumFiles to enumerate all finds and try to locate strFindFile
'
'   These dims cannot be static as they must
'   be reallocated when the proc calls itself
Dim hSearch As Long
Dim blnFoldersPresent As Boolean
Dim strFile As String
Dim strFullPath As String
Dim blnDone As Boolean
Dim blnCancel As Boolean
Dim i As Integer, intFolderCount As Integer
Dim astrFolders() As String

    intFolderCount = 0
    ReDim astrFolders(intFolderCount)
    
    '   Notify clients of our progress
    RaiseEvent SearchStatus(strPath, blnCancel)
    '  DoEvents is necessary to let the client do
    '  some processing (which includes Cancel notifications)
    DoEvents
    '   if the client set the ByRef Cancel flag to true,
    '   then terminate
    If Not blnCancel Then
        '   get a seach handle
        hSearch = apiFindFirstFile(strPath & "*", lpDataFolder)
        If Not hSearch = INVALID_HANDLE_VALUE Then
            '   if a valid handle was returned, go through all the folders
            '   till there are no more (indicated by blnFoldersPresent)
            Do While Not blnFoldersPresent
                strFile = fStripNull(lpDataFolder.cFileName)
                '   build the complete path to the file
                strFullPath = strPath & strFile
                '   we're only interested in Directories
                If apiGetFileAttributes(strFullPath) And _
                                FILE_ATTRIBUTE_DIRECTORY Then
                    '   ignore the "." (current) and ".." (parent)" entries
                    '   in a folder.  Using Asc gives slightly better performance
                    If Not (Asc(strFile) = vbKeyDot) Then
                        If mblnIncludeSubFolders Then
                            astrFolders(intFolderCount) = strFile & "\"
                            intFolderCount = intFolderCount + 1
                            ReDim Preserve astrFolders(intFolderCount)
                        End If
                    End If
                End If
                
                If apiFindNextFile(hSearch, lpDataFolder) = 0 _
                        And Err.LastDllError = ERROR_NO_MORE_FILES Then
                    '   If no more folders are present, prepare to quit the loop
                    blnFoldersPresent = True
                End If
            Loop
            Call apiFindClose(hSearch)
            '   Enumerate the files in this folder
            Call sEnumFiles(strPath, strFindFile)
            If intFolderCount > 0 Then
                For i = 0 To intFolderCount - 1
                    Call sGetFolders(strPath & astrFolders(i), strFindFile)
                Next
            End If
        End If
    End If
    
End Sub

Private Sub sEnumFiles(strPath As String, ByVal strFindFile As String)
'   Goes through all the files in a folder and if a match is found,
'   it raises the Found event and adds the filename to the internal
'   FilesFound collection
'
'   All variables are made static to save allocation time
'   this function is not recursive, but is called once for each folder
'   sGetFolders proc finds
'
Static sblnFilesPresent As Boolean
Static clsFile As File
Static sastrFiles() As String
Static sintPos As Integer
Static sintI As Integer
Static sintDim As Integer
Static sstrTemp As String
Static slngRet As Long
Const conDELIM_COMMA = ","
Const conDELIM_SEMICOLON = ";"

    '   Initialize the static variables
    sintPos = 0: sblnFilesPresent = False:  sintI = 0
    sintDim = 0:    sstrTemp = vbNullString
    Set clsFile = Nothing
    
    '   Check to see if multiple filenames were passed
    '   start with Semicolons
    sintPos = InStr(strFindFile, conDELIM_SEMICOLON)
    If sintPos > 0 Then
        '   if a semicolon is present, initialize our array
        Erase sastrFiles
        sastrFiles = Split(strFindFile, conDELIM_SEMICOLON)
        sintDim = UBound(sastrFiles)
        strFindFile = "*.*"
    Else
        '   Semicolon was not present, check for Commas
        sintPos = InStr(strFindFile, conDELIM_COMMA)
        If sintPos > 0 Then
            Erase sastrFiles
            sastrFiles = Split(strFindFile, conDELIM_COMMA)
            sintDim = UBound(sastrFiles)
            strFindFile = "*.*"
        End If
    End If
    
    '   obtain a search handle
    hSearchFile = apiFindFirstFile(strPath & strFindFile, lpDataFile)
    If Not hSearchFile = INVALID_HANDLE_VALUE Then
        '   if a valid handle was returned, go through all the files
        '   till there are no more (indicated by blnFilespresent)
        Do While Not sblnFilesPresent
            '   only look at files
            sstrTemp = fStripNull(lpDataFile.cFileName)
            If Not (Asc(sstrTemp) = vbKeyDot) Then
                slngRet = apiGetFileAttributes(strPath & sstrTemp)
                If (slngRet And FILE_ATTRIBUTE_DIRECTORY) = 0 Then
                    If sintDim > 0 Then
                        '   if the internal filename arrays is initialized, that means
                        '   multiple filenames were passed
                        '   go through the entire array to see if a match for the filename
                        '   is present in this folder.  One array element can be present only once
                        For sintI = 0 To sintDim
                            If StrComp(sastrFiles(sintI), sstrTemp, vbTextCompare) = 0 Then
                                '   if the filenames match, add the file to the collection
                                Set clsFile = mclsFiles.Add(VarPtr(lpDataFile), strPath)
                                '   and notify the clients that a file was found
                                RaiseEvent Found(clsFile)
                            End If
                        Next
                    Else
                        '   Control pops here if there's only one filename specified
                        '   So add it to the collection
                        Set clsFile = mclsFiles.Add(VarPtr(lpDataFile), strPath)
                        '   and notify the clients
                        RaiseEvent Found(clsFile)
                    End If
                Else
                    ' Check for error code = 32 (in use by another process)
                    If Err.LastDllError = 32 And slngRet = -1 Then
                        ' let's add the file anyway
                        Set clsFile = mclsFiles.Add(VarPtr(lpDataFile), strPath)
                        '   and notify the clients
                        RaiseEvent Found(clsFile)
                    End If
                End If
            End If
            If apiFindNextFile(hSearchFile, lpDataFile) = 0 _
                    And Err.LastDllError = ERROR_NO_MORE_FILES Then
                sblnFilesPresent = True
            End If
        Loop
        Call apiFindClose(hSearchFile)
        hSearchFile = 0
    End If
End Sub

Private Function fIsSameStr(lngPtr1 As Long, lngPtr2 As Long) As Boolean
    fIsSameStr = (apiCompareString(LOCALE_SYSTEM_DEFAULT, _
                    NORM_IGNORECASE, lngPtr1, -1, lngPtr2, -1) = CSTR_EQUAL)
End Function

Private Function fIsLocalFixedDrive(ByVal strDrive As String) As Boolean
'   Returns true if the drive is a local fixed drive
'
    Select Case apiGetDriveType(strDrive)
        Case DRIVE_FIXED:   fIsLocalFixedDrive = True
    End Select
End Function

Private Function fGetDrives() As String
'   Returns all the installed local drive letters as a
'   vbNullChar separated string
Dim lpBuffer As String
Dim lngRet As Long
    lpBuffer = Space$(MAX_PATH + 1)
    lngRet = apiGetLogicalDriveStrings(MAX_PATH, lpBuffer)
    If lngRet > MAX_PATH Then
        lpBuffer = Space$(lngRet + 1)
        lngRet = apiGetLogicalDriveStrings(lngRet, lpBuffer)
    End If
    fGetDrives = Left$(lpBuffer, lngRet - 1)
End Function

Private Function fStripNull(str As String) As String
'   Returns the string after truncating it at first vbNullChar
Static intPos As Integer
    intPos = 0
    intPos = InStr(str, vbNullChar)
    If intPos > 0 Then
        fStripNull = Left$(str, intPos - 1)
    End If
End Function

Private Function fAPIErr(ByVal lngErr As Long) As String
'   Returns the description associated with Err.LastDLLError
'
Dim strMsg As String
Dim lngRet As Long
    strMsg = String$(1024, 0)
    lngRet = apiFormatMsgLong(FORMAT_MESSAGE_FROM_SYSTEM, 0, _
                           lngErr, 0, strMsg, Len(strMsg), ByVal 0)
    If lngRet Then
        fAPIErr = Left$(strMsg, lngRet)
    End If
End Function

Private Sub Class_Terminate()
    Set mclsFiles = Nothing
End Sub


